#include <stdio.h>
#include <string.h>
#include <math.h>
#include "stm32f10x.h"
#include "ad8302.h"

extern void Sleep(uint32_t nms);

//Connection of AD8302 to ADC:
//Vref - PC2
//Vmag - PC0
//Vpha - PC1

#define _PI 3.1415926

const float PI = _PI;

static UADC AD8302_MidUADC(uint32_t num, UADC *pADC);

UADC AD8302_ReadADCOnce(void)
{
    UADC res = {0};

    ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 1, ADC_SampleTime_13Cycles5);
    ADC_SoftwareStartConvCmd(ADC1, ENABLE);
    //_delayUs(50);
    while(RESET == ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
    res.valUab = ADC_GetConversionValue(ADC1);

    ADC_RegularChannelConfig(ADC1, ADC_Channel_11, 1, ADC_SampleTime_13Cycles5);
    ADC_SoftwareStartConvCmd(ADC1, ENABLE);
    while(RESET == ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
    //_delayUs(100);
    res.valPhase = ADC_GetConversionValue(ADC1);

    ADC_RegularChannelConfig(ADC1, ADC_Channel_12, 1, ADC_SampleTime_13Cycles5);
    ADC_SoftwareStartConvCmd(ADC1, ENABLE);
    //_delayUs(100);
    while(RESET == ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
    res.valUref = ADC_GetConversionValue(ADC1);
    return res;
}

UADC AD8302_ReadADC(void)
{
    static const uint32_t num = 5;
    static UADC results[10];
    uint32_t i;
    for(i = 0; i < num; i++)
    {
        results[i] = AD8302_ReadADCOnce();
    }
    return AD8302_MidUADC(num, results);
}

static inline uint32_t minu(a1, a2)
{
    return a1 < a2 ? a1 : a2;
}
static inline uint32_t maxu(a1, a2)
{
    return a1 > a2 ? a1 : a2;
}

static UADC AD8302_MidUADC(uint32_t num, UADC *pADC)
{
    uint32_t i;

    UADC res = {0};
    if (num < 3)
        return res;

    UADC resMax = {0};
    UADC resMin = {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF};

    //find min and max values
    for(i = 0; i < num; i++)
    {
        UADC* ptr = &pADC[i];
        resMax.valUref = maxu(resMax.valUref, ptr->valUref);
        resMin.valUref = minu(resMin.valUref, ptr->valUref);
        resMax.valUab = maxu(resMax.valUab, ptr->valUab);
        resMin.valUab = minu(resMin.valUab, ptr->valUab);
        resMax.valPhase = maxu(resMax.valPhase, ptr->valPhase);
        resMin.valPhase = minu(resMin.valPhase, ptr->valPhase);
    }
    //calculate average excluding min and max values
    int minExcluded = 0;
    int maxExcluded = 0;
    for (i = 0; i < num; i++)
    {
        UADC* ptr = &pADC[i];
        if (!minExcluded && ptr->valUab == resMin.valUab)
        {
            minExcluded = 1;
            continue;
        }
        if (!maxExcluded && ptr->valUab == resMax.valUab)
        {
            maxExcluded = 1;
            continue;
        }
        res.valUab += ptr->valUab;
    }
    res.valUab /= (num - 2);

    minExcluded = 0;
    maxExcluded = 0;
    for (i = 0; i < num; i++)
    {
        UADC* ptr = &pADC[i];
        if (!minExcluded && ptr->valPhase == resMin.valPhase)
        {
            minExcluded = 1;
            continue;
        }
        if (!maxExcluded && ptr->valPhase == resMax.valPhase)
        {
            maxExcluded = 1;
            continue;
        }
        res.valPhase += ptr->valPhase;
    }
    res.valPhase /= (num - 2);

    minExcluded = 0;
    maxExcluded = 0;
    for (i = 0; i < num; i++)
    {
        UADC* ptr = &pADC[i];
        if (!minExcluded && ptr->valUref == resMin.valUref)
        {
            minExcluded = 1;
            continue;
        }
        if (!maxExcluded && ptr->valUref == resMax.valUref)
        {
            maxExcluded = 1;
            continue;
        }
        res.valUref += ptr->valUref;
    }
    res.valUref /= (num - 2);
    return res;
}

float AD8302_GetPhaseDegrees(UADC in)
{
    if(in.valPhase > in.valUref)
    {
        in.valPhase = in.valUref;
    }
    float phi = PI - ((float)in.valPhase / (float)in.valUref) * PI;
    return (phi * 180.0) / PI;
}

float AD8302_GetPhaseRadians(UADC in)
{
    if(in.valPhase > in.valUref)
    {
        in.valPhase = in.valUref;
    }
    float phi = PI - ((float)in.valPhase / (float)in.valUref) * PI;
    return phi;
}

float AD8302_GetRatioDb(UADC in)
{
    if(in.valUab > in.valUref)
    {
        in.valUab = in.valUref;
    }
    float mid = ((float)in.valUref) / 2.0;
    float oneDb = mid / 30.0;
    float diff = (float)in.valUab - mid;
    float diffDb = diff / oneDb;
    return -diffDb;
}

float AD8302_GetRatio(UADC in)
{
    float dB = AD8302_GetRatioDb(in);
    return powf(10.0, dB * 0.05);
}

void AD8302_Init(void)
{
    GPIO_InitTypeDef  GPIO_InitStructure;
    ADC_InitTypeDef   ADC_InitStructure;
    //DMA_InitTypeDef DMA_InitStructure;

    // PCLK2 is the APB2 clock
    // ADCCLK = PCLK2/6 = 72/6 = 12MHz
    RCC_ADCCLKConfig(RCC_PCLK2_Div8);

    // Enable GPIOA, GPIOB, GPIOC, GPIOD, GPIOE and AFIO clocks
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO | RCC_APB2Periph_ADC1, ENABLE);

    // Enable PWR and BKP clock
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

    GPIO_StructInit(&GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);

    /*
    //DMA for ADC
    DMA_DeInit(DMA1_Channel1);
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&(ADC1->DR);
    DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)ADCResults;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_BufferSize = 3;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel1, &DMA_InitStructure);
    // Enable DMA1 Channel1
    DMA_Cmd(DMA1_Channel1, ENABLE);
    */

    //Enable ADC on PC0, PC1, PC2
    //(ADC Channel10, ADC Channel11 and ADC Channel12)
    ADC_DeInit(ADC1);
//   ADC_Cmd(ADC1, DISABLE);
    GPIO_StructInit(&GPIO_InitStructure);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    ADC_Cmd(ADC1, ENABLE);

    // ADC1 regular channels configuration
    ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 1, ADC_SampleTime_13Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_11, 2, ADC_SampleTime_13Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_12, 3, ADC_SampleTime_13Cycles5);

    ADC_StructInit(&ADC_InitStructure);
    ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
    ADC_InitStructure.ADC_ScanConvMode = DISABLE;
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
    ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_NbrOfChannel = 1;
    ADC_Init(ADC1, &ADC_InitStructure);

    //ADC_DMACmd(ADC1, ENABLE);

    ADC_ResetCalibration(ADC1);
    while(ADC_GetResetCalibrationStatus(ADC1));
    ADC_StartCalibration(ADC1);
    while(ADC_GetCalibrationStatus(ADC1));

}

