#include "stm32f10x.h"
#include "dds.h"


//DDS AD9851 is connected to:
//FQ_UD - PB12
//CLK - PB14
//DATA - PB15
//RESET - PB12

#define DDS_FQ_UD GPIO_Pin_12
#define DDS_CLK   GPIO_Pin_14
#define DDS_DATA  GPIO_Pin_8
#define DDS_RST   GPIO_Pin_3
#define DDS_GPIO  GPIOB
#define DDS_GPIO_RST GPIOD

//25067.6
//12533.84
#define SCLK (30000000ULL * 6ULL)

static uint32_t dds_freq = 14000 * 1000;
static uint32_t dds_isInit = 0;

static inline void _delayUs(uint32_t us)
{
    //Calibrated for 72 MHz and code in flash
    asm volatile("           mov   r2, #14    \n"
                 "           mov   r1, #0     \n"
                 "2:         cmp   r1, r2     \n"
                 "           itt   lo         \n"
                 "           addlo r1, r1, #1 \n"
                 "           blo   2b         \n"
                 "           mov   r1, #8     \n"
                 "           mul   r2, %0, r1 \n"
                 "           mov   r1, #0     \n"
                 "1:         nop              \n"
                 "           cmp   r1, r2     \n"
                 "           itt   lo         \n"
                 "           addlo r1, r1, #1 \n"
                 "           blo   1b  \n"::"r"(us):"r1", "r2");
}

static void dds_Delay(void)
{
    _delayUs(2);
}

static void dds_setFreq(void)
{
    uint32_t fword;
    uint32_t mask = 1UL;

    fword = (uint32_t)((((uint64_t)dds_freq) << 32) / SCLK);
    while(mask)
    {
        if(0 != (fword & mask))
        {
            GPIO_SetBits(DDS_GPIO, DDS_DATA);
        }
        else
        {
            GPIO_ResetBits(DDS_GPIO, DDS_DATA);
        }
        dds_Delay();
        GPIO_SetBits(DDS_GPIO, DDS_CLK);
        dds_Delay();
        GPIO_ResetBits(DDS_GPIO, DDS_CLK);
        dds_Delay();
        mask <<= 1;
    }
    mask = 1UL;
    fword = 1UL;
    while(mask < 256)
    {
        if(0 != (fword & mask))
        {
            GPIO_SetBits(DDS_GPIO, DDS_DATA);
        }
        else
        {
            GPIO_ResetBits(DDS_GPIO, DDS_DATA);
        }
        dds_Delay();
        GPIO_SetBits(DDS_GPIO, DDS_CLK);
        dds_Delay();
        GPIO_ResetBits(DDS_GPIO, DDS_CLK);
        dds_Delay();
        mask <<= 1;
    }
    dds_Delay();
    dds_Delay();
    GPIO_SetBits(DDS_GPIO, DDS_FQ_UD);
    dds_Delay();
    GPIO_ResetBits(DDS_GPIO, DDS_FQ_UD);
    _delayUs(500);
}

//==============================================================================
//Initialize DDS:
// configure GPIO, reset, set serial mode, set CLK multiplier x6,
// set frequency
//==============================================================================
void DDS_Init(void)
{
    GPIO_InitTypeDef  GPIO_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD | RCC_APB2Periph_GPIOB, ENABLE);

    GPIO_InitStructure.GPIO_Pin =  DDS_FQ_UD | DDS_CLK | DDS_DATA;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(DDS_GPIO, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin =  DDS_RST;
    GPIO_Init(DDS_GPIO_RST, &GPIO_InitStructure);

    GPIO_SetBits(DDS_GPIO_RST, DDS_RST);
    dds_Delay();
    GPIO_ResetBits(DDS_GPIO, DDS_FQ_UD | DDS_CLK | DDS_DATA);
    dds_Delay();
    GPIO_ResetBits(DDS_GPIO_RST, DDS_RST);
    dds_Delay();
    dds_Delay();
    dds_Delay();

    GPIO_SetBits(DDS_GPIO, DDS_CLK);
    dds_Delay();
    GPIO_ResetBits(DDS_GPIO, DDS_CLK);
    dds_Delay();
    GPIO_SetBits(DDS_GPIO, DDS_FQ_UD);
    dds_Delay();
    GPIO_ResetBits(DDS_GPIO, DDS_FQ_UD);

    dds_Delay();
    dds_Delay();
    dds_Delay();
    dds_Delay();
    dds_Delay();

    dds_setFreq();

    dds_isInit = 1;
} // void DDS_Init(void)

//Returns current DDS frequency
uint32_t DDS_GetFreq(void)
{
    if(!dds_isInit)
    {
        return 0;
    }
    return dds_freq;
}

//Sets new DDS frequency in Hz
void DDS_SetFreq(uint32_t freq)
{
    if(!dds_isInit)
    {
        DDS_Init();
    }

    dds_freq = freq;
    dds_setFreq();
}
